==> xwidgetsbase-8.2.8.tar.gz: lib/XfXWidgets/XElenco.dart <==
import 'package:flutter/material.dart';
import 'package:uuid/uuid.dart';
import 'package:xwidgetsbase/XStylesBase.dart';

import '../xWidgetsBase.dart';

typedef ItemCreator<S> = S Function();

class XElenco<T extends XFDataItem> extends StatefulWidget {
  XElenco(
    this.list, {
    this.title,
    this.title_Widget,
    this.list_Item_FullWidget,
    this.list_Item_CoreWidget,
    this.sortingArea_Widget,
    this.item_propToShow = "",
    this.list_propToFilter = "",
    this.cmdItem_Add,
    this.cmdItem_Add_Widget,
    this.cmdItem_Delete,
    this.cmdItem_Open,
    this.cmdItem_Opened_Widget,
    this.filterForList,
    this.xThemeData,
    this.itemCreator, //
    this.alert_title,
    this.alert_NameOfObjToDelete,
    this.alert_backGroundColor,
    this.alert_label_NO,
    this.alert_label_YES,
    this.alert_onPressed_NO,
    this.alert_onPressed_YES,
    this.alert_title_Style,
    Key? key,
  }) : super(key: key);

  final String? title;
  final Widget? title_Widget;
  final List<T> list;
  final Widget? sortingArea_Widget;
  final ThemeData? xThemeData;

  ///Solo widget interno (def: Text). riamgno i btn e gestisce onTap e onDel
  Widget Function(T)? list_Item_CoreWidget;
  final String? item_propToShow;
  final String? list_propToFilter;
  final Function()? cmdItem_Add;
  final Widget? cmdItem_Add_Widget;

  final Function(T)? cmdItem_Delete;
  final Function(T)? cmdItem_Open;
  final int? filterForList;

  ///W con il quale si visualizzerà il dettaglio dell'item quando lo U darà il coamndo open (onTap)
  Widget Function(T, Function())? cmdItem_Opened_Widget;

  ///sost tutta la riga compreso di onTap
  Widget Function(T)? list_Item_FullWidget;
  //gestione ItemOpen: apro in me stesso il widget cmdItemOpen_widget

  ItemCreator<T>? itemCreator;

  String? alert_title;
  TextStyle? alert_title_Style;
  Color? alert_backGroundColor;
  String? alert_label_YES;
  String? alert_label_NO;
  Function()? alert_onPressed_YES;
  Function()? alert_onPressed_NO;
  String? alert_NameOfObjToDelete;

  @override
  _XElencoState<T> createState() => _XElencoState<T>();
}

class _XElencoState<T extends XFDataItem> extends State<XElenco<T>> {
  T? itemInEdit;
  _cmdAdd() {
    if (widget.cmdItem_Opened_Widget != null) {
      setState(() {
        itemInEdit = widget.itemCreator!();
      });
    }
  }

  _cmdDeleteItem(T item) async {
    var res = await alertForDeleteItemFromList(
      context,
      widget.alert_NameOfObjToDelete ?? ("the " + item.X_nome! + " '" + item[widget.item_propToShow!]! + "'"),
      backGroundColor: widget.alert_backGroundColor,
      label_NO: widget.alert_label_NO,
      label_YES: widget.alert_label_YES,
      onPressed_NO: widget.alert_onPressed_NO,
      onPressed_YES: widget.alert_onPressed_YES,
      title: widget.alert_title,
      title_Style: widget.alert_title_Style,
    );
    if (res) {
      setState(() {
        widget.list.remove(item);
      });
      await widget.cmdItem_Delete != null ? widget.cmdItem_Delete!(item) : () {};
    }
  }

  _cmdOnTapItem(T item) {
    if (widget.cmdItem_Opened_Widget != null) {
      setState(() {
        itemInEdit = item;
      });
    }
  }

  Future<bool> _onWillPop() async {
    if (itemInEdit == null) {
      return true;
    } else {
      if (itemInEdit!.id == "") {
        itemInEdit!.id = Uuid().v4();
        //ADD NEW
        widget.list.add(itemInEdit!);
      }
      setState(() {
        itemInEdit = null;
      });
      return false;
    }
  }

  pippp() async {
    await _onWillPop();
  }

  @override
  Widget build(BuildContext context) {
    if (itemInEdit != null) {
      return WillPopScope(
          onWillPop: _onWillPop,
          child: Column(mainAxisSize: MainAxisSize.min, children: [
            widget.cmdItem_Opened_Widget == null
                ? Container(child: Text("sticazzi")) //
                : widget.cmdItem_Opened_Widget!(itemInEdit!, pippp)
          ]));
    } else {
      return Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(height: 10),
          //title
          Container(
              margin: EdgeInsets.symmetric(horizontal: 30),
              child: Column(children: [
                widget.title == null
                    ? Container()
                    : Container(
                        margin: EdgeInsets.symmetric(horizontal: 20),
                        padding: EdgeInsets.only(bottom: 5),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [Expanded(child: widget.title_Widget ?? Text(widget.title ?? "", textAlign: TextAlign.center, style: XStyles.xStyTextForLargeTitle()))],
                        )),
              ])),
          widget.list.isNotEmpty ? widget.sortingArea_Widget ?? Container() : Container(),
          widget.list.isNotEmpty ? Divider(height: 4) : Container(),
          Flexible(
              child: SingleChildScrollView(
                  child: Container(
                      margin: EdgeInsets.symmetric(horizontal: 30),
                      child: Column(
                          mainAxisSize: MainAxisSize.min,
                          children: widget.list
                              .where((element) => widget.filterForList == 0 ? true : ((element as dynamic)[widget.list_propToFilter!] == widget.filterForList))
                              .toList()
                              .map(widget.list_Item_FullWidget != null
                                  ? widget.list_Item_FullWidget!
                                  : (item) {
                                      return GestureDetector(
                                          onTap: widget.cmdItem_Open != null
                                              ? () => widget.cmdItem_Open!(item) //
                                              : widget.cmdItem_Opened_Widget == null
                                                  ? null
                                                  : () => _cmdOnTapItem(item),
                                          child: widget.list_Item_CoreWidget != null
                                              ? widget.list_Item_CoreWidget!(item)
                                              : Container(
                                                  margin: EdgeInsets.symmetric(vertical: 5),
                                                  child: Column(children: [
                                                    SizedBox(height: 10),
                                                    Row(children: [
                                                      Expanded(
                                                          child: Container(
                                                              color: widget.xThemeData!.colorScheme.background,
                                                              height: 40,
                                                              alignment: Alignment.centerLeft,
                                                              child: Text(
                                                                widget.item_propToShow == null ? " " : (" " + (item as dynamic)[widget.item_propToShow!]),
                                                                textAlign: TextAlign.center,
                                                                style: widget.xThemeData!.textTheme.bodyText2!.apply(fontSizeDelta: 11),
                                                              ))),
                                                      XBtnbaseForTheme(
                                                        widthBTN: 50,
                                                        heightBTN: 50,
                                                        elevation: 0,
                                                        iconSize: 35,
                                                        icon: Icons.delete,
                                                        iconColor: widget.xThemeData!.colorScheme.onPrimary,
                                                        onPressed: widget.cmdItem_Delete == null ? () => _cmdDeleteItem(item) : () => {widget.cmdItem_Delete!(item)},
                                                        colorBTN: Colors.transparent,
                                                      ),
                                                    ]),
                                                    Divider(color: widget.xThemeData!.colorScheme.onPrimary, height: 10, thickness: 2),
                                                  ])));
                                    })
                              .toList())))),
          Column(children: [
            Divider(),
            //SizedBox(height: 10),
            Row(mainAxisAlignment: MainAxisAlignment.center, children: [
              Expanded(
                  child: widget.cmdItem_Add_Widget ??
                      XBtnbaseForTheme.onlyBorderAndLabel(
                        marginBTN: EdgeInsets.symmetric(horizontal: 30),
                        colorBorder: widget.xThemeData!.colorScheme.primary,
                        colorBackGround: Colors.transparent,
                        iconColor: widget.xThemeData!.colorScheme.onPrimary,
                        icon: Icons.add,
                        onPressed: widget.cmdItem_Add ?? () => _cmdAdd(),
                      )),
            ]),
            SizedBox(height: 30),
          ])
        ],
      );
    }
  }

  Future<bool> alertForDeleteItemFromList(
    BuildContext context,
    String nameOfObjToDelete, {
    String? title,
    TextStyle? title_Style,
    Color? backGroundColor,
    String? label_YES,
    String? label_NO,
    Function()? onPressed_YES,
    Function()? onPressed_NO,
  }) async {
    var x = await showDialog<bool>(
        context: context,
        builder: (context) {
          return XAlertDialog(
            titleText: title ?? "Are you sure to delete ${nameOfObjToDelete} from the list?",
            styleTitle: title_Style ?? XStyles.xStyTextForLabel(widget.xThemeData!.colorScheme.primary),
            textAlignTitle: TextAlign.center,
            colorBackGround: backGroundColor ?? widget.xThemeData!.colorScheme.onBackground,
            labelBTNyes: label_YES ?? "Yes",
            onPressedYes: onPressed_YES ?? () => Navigator.pop(context, true),
            labelBTNno: label_NO ?? "No",
            onPressedNo: onPressed_NO ?? () => Navigator.pop(context, false),
          );
        });

    if (x != null && x == true) {
      return true;
    } else {
      return false;
    }
  }
}

@override
class XFDataItem {
  static const String id_ColName = 'id';
  static const String id_ColLabel = 'id';

  /// 'id'
  String? id;
  bool? isSel;
  bool? isShowDetails;
  String? get X_nome => '';
  String? get X_nomeMF => '';
  String? get X_nomePlurale => '';
  String? get X_docCaption => '';

  XFDataItem({this.id});

  operator [](String key) {
    return getProp(key);
  }

  dynamic getProp(String key) {}
  copyProps(dynamic objectFrom, dynamic objTO) {}

  XFDataItem getCloneFull() {
    var r = XFDataItem();
    r.copyProps(this, r);
    return r;
  }
}

==> xwidgetsbase-8.3.0.tar.gz: lib/XfXWidgets/XElenco.dart <==
// ignore_for_file: must_be_immutable

import 'package:flutter/material.dart';
import 'package:xwidgetsbase/XStylesBase.dart';

import '../xWidgetsBase.dart';

typedef ItemCreator<S> = S Function();

class XElenco<T> extends StatefulWidget {
  XElenco(
    this.list, {
    this.title,
    this.title_Widget,
    this.list_Item_FullWidget,
    this.list_Item_CoreWidget,
    this.sortingArea_Widget,
    this.item_propToShow = "",
    this.list_propToFilter = "",
    this.cmdItem_Add,
    this.cmdItem_Add_Widget,
    this.cmdItem_Delete,
    this.cmdItem_Open,
    this.cmdItem_Opened_Widget,
    this.filterForList,
    this.xThemeData,
    this.itemCreator,
    this.alert_title,
    this.alert_NameOfObjToDelete,
    this.alert_backGroundColor,
    this.alert_label_NO,
    this.alert_label_YES,
    this.alert_onPressed_NO,
    this.alert_onPressed_YES,
    this.alert_title_Style,
    Key? key,
  }) : super(key: key);

  final String? title;
  final Widget? title_Widget;
  final List<T> list;
  final Widget? sortingArea_Widget;
  final ThemeData? xThemeData;

  ///Solo widget interno (def: Text). riamgno i btn e gestisce onTap e onDel
  Widget Function(T)? list_Item_CoreWidget;
  final String? item_propToShow;
  final String? list_propToFilter;
  final Function()? cmdItem_Add;
  final Widget? cmdItem_Add_Widget;

  final Function(T)? cmdItem_Delete;
  final Function(T)? cmdItem_Open;
  final int? filterForList;

  ///W con il quale si visualizzerà il dettaglio dell'item quando lo U darà il coamndo open (onTap)
  Widget Function(T, Function())? cmdItem_Opened_Widget;

  ///sost tutta la riga compreso di onTap
  Widget Function(T)? list_Item_FullWidget;
  //gestione ItemOpen: apro in me stesso il widget cmdItemOpen_widget

  ItemCreator<T>? itemCreator;

  String? alert_title;
  TextStyle? alert_title_Style;
  Color? alert_backGroundColor;
  String? alert_label_YES;
  String? alert_label_NO;
  Function()? alert_onPressed_YES;
  Function()? alert_onPressed_NO;
  String? Function(T)? alert_NameOfObjToDelete;

  @override
  _XElencoState<T> createState() => _XElencoState<T>();
}

class _XElencoState<T> extends State<XElenco<T>> {
  T? itemInEdit;
  _cmdAdd() {
    if (widget.cmdItem_Opened_Widget != null) {
      setState(() {
        itemInEdit = widget.itemCreator!();
      });
    }
  }

  _cmdDeleteItem(T item) async {
    var res = await alertForDeleteItemFromList(
      context,
      widget.alert_NameOfObjToDelete!(item) ?? "",
      backGroundColor: widget.alert_backGroundColor,
      label_NO: widget.alert_label_NO,
      label_YES: widget.alert_label_YES,
      onPressed_NO: widget.alert_onPressed_NO,
      onPressed_YES: widget.alert_onPressed_YES,
      title: widget.alert_title,
      title_Style: widget.alert_title_Style,
    );
    if (res) {
      setState(() {
        widget.list.remove(item);
      });
      await widget.cmdItem_Delete != null ? widget.cmdItem_Delete!(item) : () {};
    }
  }

  _cmdOnTapItem(T item) {
    if (widget.cmdItem_Opened_Widget != null) {
      setState(() {
        itemInEdit = item;
      });
    }
  }

  Future<bool> _onWillPop() async {
    if (itemInEdit == null) {
      return true;
    } else {
      // if (itemInEdit!.id == "") {
      //   itemInEdit!.id = Uuid().v4();
      //   //ADD NEW
      //   widget.list.add(itemInEdit!);
      // }
      setState(() {
        itemInEdit = null;
      });
      return false;
    }
  }

  pippp() async {
    await _onWillPop();
  }

  @override
  Widget build(BuildContext context) {
    if (itemInEdit != null) {
      return WillPopScope(
          onWillPop: _onWillPop,
          child: Column(mainAxisSize: MainAxisSize.min, children: [
            widget.cmdItem_Opened_Widget == null
                ? Container(child: Text("sticazzi")) //
                : widget.cmdItem_Opened_Widget!(itemInEdit!, pippp)
          ]));
    } else {
      return Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(height: 10),
          //title
          Container(
              margin: EdgeInsets.symmetric(horizontal: 30),
              child: Column(children: [
                widget.title == null
                    ? Container()
                    : Container(
                        margin: EdgeInsets.symmetric(horizontal: 20),
                        padding: EdgeInsets.only(bottom: 5),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [Expanded(child: widget.title_Widget ?? Text(widget.title ?? "", textAlign: TextAlign.center, style: XStyles.xStyTextForLargeTitle()))],
                        )),
              ])),
          widget.list.isNotEmpty ? widget.sortingArea_Widget ?? Container() : Container(),
          widget.list.isNotEmpty ? Divider(height: 4) : Container(),
          Flexible(
              child: SingleChildScrollView(
                  child: Container(
                      margin: EdgeInsets.symmetric(horizontal: 30),
                      child: Column(
                          mainAxisSize: MainAxisSize.min,
                          children: widget.list
                              .where((element) => widget.filterForList == 0 ? true : ((element as dynamic)[widget.list_propToFilter!] == widget.filterForList))
                              .toList()
                              .map(widget.list_Item_FullWidget != null
                                  ? widget.list_Item_FullWidget!
                                  : (item) {
                                      return GestureDetector(
                                          onTap: widget.cmdItem_Open != null
                                              ? () => widget.cmdItem_Open!(item) //
                                              : widget.cmdItem_Opened_Widget == null
                                                  ? null
                                                  : () => _cmdOnTapItem(item),
                                          child: widget.list_Item_CoreWidget != null
                                              ? widget.list_Item_CoreWidget!(item)
                                              : Container(
                                                  margin: EdgeInsets.symmetric(vertical: 5),
                                                  child: Column(children: [
                                                    SizedBox(height: 10),
                                                    Row(children: [
                                                      Expanded(
                                                          child: Container(
                                                              color: widget.xThemeData!.colorScheme.background,
                                                              height: 40,
                                                              alignment: Alignment.centerLeft,
                                                              child: Text(
                                                                widget.item_propToShow == null ? " " : (" " + (item as dynamic)[widget.item_propToShow!]),
                                                                textAlign: TextAlign.center,
                                                                style: widget.xThemeData!.textTheme.bodyText2!.apply(fontSizeDelta: 11),
                                                              ))),
                                                      XBtnbaseForTheme(
                                                        widthBTN: 50,
                                                        heightBTN: 50,
                                                        elevation: 0,
                                                        iconSize: 35,
                                                        icon: Icons.delete,
                                                        iconColor: widget.xThemeData!.colorScheme.onPrimary,
                                                        onPressed: widget.cmdItem_Delete == null ? () => _cmdDeleteItem(item) : () => {widget.cmdItem_Delete!(item)},
                                                        colorBTN: Colors.transparent,
                                                      ),
                                                    ]),
                                                    Divider(color: widget.xThemeData!.colorScheme.onPrimary, height: 10, thickness: 2),
                                                  ])));
                                    })
                              .toList())))),
          Column(children: [
            Divider(),
            //SizedBox(height: 10),
            Row(mainAxisAlignment: MainAxisAlignment.center, children: [
              Expanded(
                  child: widget.cmdItem_Add_Widget ??
                      XBtnbaseForTheme.onlyBorderAndLabel(
                        marginBTN: EdgeInsets.symmetric(horizontal: 30),
                        colorBorder: widget.xThemeData!.colorScheme.primary,
                        colorBackGround: Colors.transparent,
                        iconColor: widget.xThemeData!.colorScheme.onPrimary,
                        icon: Icons.add,
                        onPressed: widget.cmdItem_Add ?? () => _cmdAdd(),
                      )),
            ]),
            SizedBox(height: 30),
          ])
        ],
      );
    }
  }

  Future<bool> alertForDeleteItemFromList(
    BuildContext context,
    String nameOfObjToDelete, {
    String? title,
    TextStyle? title_Style,
    Color? backGroundColor,
    String? label_YES,
    String? label_NO,
    Function()? onPressed_YES,
    Function()? onPressed_NO,
  }) async {
    var x = await showDialog<bool>(
        context: context,
        builder: (context) {
          return XAlertDialog(
            titleText: title ?? "Are you sure to delete ${nameOfObjToDelete} from the list?",
            styleTitle: title_Style ?? XStyles.xStyTextForLabel(widget.xThemeData!.colorScheme.primary),
            textAlignTitle: TextAlign.center,
            colorBackGround: backGroundColor ?? widget.xThemeData!.colorScheme.onBackground,
            labelBTNyes: label_YES ?? "Yes",
            onPressedYes: onPressed_YES ?? () => Navigator.pop(context, true),
            labelBTNno: label_NO ?? "No",
            onPressedNo: onPressed_NO ?? () => Navigator.pop(context, false),
          );
        });

    if (x != null && x == true) {
      return true;
    } else {
      return false;
    }
  }
}

class XFDataItem<T> {
  static const String id_ColName = 'id';
  static const String id_ColLabel = 'id';

  /// 'id'
  String? id;
  bool? isSel;
  bool? isShowDetails;
  String? get X_nome => '';
  String? get X_nomeMF => '';
  String? get X_nomePlurale => '';
  String? get X_docCaption => '';

  XFDataItem({this.id});

  operator [](String key) {
    return getProp(key);
  }

  dynamic getProp(String key) {}
  copyProps(dynamic objectFrom, dynamic objTO) {}

  XFDataItem<T> getCloneFull() {
    var r = XFDataItem<T>();
    r.copyProps(this, r);
    return r;
  }
}
